/*
 * button.h
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _BUTTON_H_
#define _BUTTON_H_


class Button
{
public:

	/**
	 * Constructor.
	 *
	 * @param pin the pin to monitor
	 */
	Button( int pin );

	/**
	 * Set event times.
	 *
	 * @oaram pointer to ordered, zero-terminated array of event times
	 */
	void set_event_times( int *event_times );

	/**
	 * Set button press mode
	 *
	 * @param send_interim send interim presses during a long press?
	 */
	void set_press_mode( bool send_interim );

	/**
	 * Update internal state (call as regularly sa possible!)
	 */
	void update();

	/**
	 * Get the first pending event.
	 *
	 * @return event id, 0 if there isn't any
	 */
	int get_event();

	/**
	 * Get the current button state.
	 *
	 * @return true if the button is pressed
	 */
	bool get_state();

protected:

	/** pin */
	int _pin;

	/** last real state */
	bool _last_real_state;

	/** time when the real state began */
	unsigned long _millis_real_state;

	/** accepted state */
	bool _state;

	/** time when the accepted state began */
	unsigned long _millis_state;

	/** amount of time that we've already triggered events for this press */
	unsigned long _millis_done;

	/** pointer to event times */
	int *_event_times;

	/** pending event */
	int _pending_event;
};


#endif //_BUTTON_H_
