/*
 * analogue_clock_mode.cc
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "analogue_clock_mode.h"
#include "config.h"
#include "Arduino.h"
#include "time.h"
#include "common.h"
#include "text.h"


void AnalogueClockMode::draw( int segment )
{
	bool draw_tick = segment && ( segment % ( NUM_SECOND_SEGMENTS * 5 ) == 0 ||
		segment == 1 || segment == NUM_SEGMENTS - 1 );
	bool draw_hour = segment == _hour_segment;
	bool draw_hour_side = 2 >=
		( abs( _hour_segment - segment ) + NUM_SEGMENTS ) % NUM_SEGMENTS;
	bool draw_minute = segment == _minute_segment;
	bool draw_minute_side = 1 >=
		( abs( _minute_segment - segment ) + NUM_SEGMENTS ) % NUM_SEGMENTS;

	bool draw_second = segment == _second_segment;

	// set the LEDs
	switch( _flavour )
	{
	case 0:
		led( 9, true );
		led( 8, draw_tick || draw_second );
		led( 7, draw_minute || draw_second );
		led( 6, draw_minute || draw_minute_side || draw_second );
		led( 5, draw_minute || draw_minute_side ||
					  draw_second || draw_hour );
		for( int a = 0; a <= 4; a++ )
			led( a, draw_minute || draw_minute_side ||
						  draw_second || draw_hour || draw_hour_side );
		break;
	case 1:
		led( 9, draw_tick );
		led( 8, draw_second );
		led( 7, draw_minute || draw_second );
		led( 6, draw_minute || draw_minute_side || draw_second );
		led( 5, draw_minute || draw_minute_side ||
					  draw_second || draw_hour );
		for( int a = 0; a <= 4; a++ )
			led( a, draw_minute || draw_minute_side ||
						  draw_second || draw_hour || draw_hour_side );
		break;
	case 2:
		led( 9, segment <= _subsecond_segment );
		led( 8, draw_tick || draw_second );
		led( 7, draw_minute || draw_second );
		led( 6, draw_minute || draw_minute_side || draw_second );
		led( 5, draw_minute || draw_minute_side ||
					  draw_second || draw_hour );
		for( int a = 0; a <= 4; a++ )
			led( a, draw_minute || draw_minute_side ||
						  draw_second || draw_hour || draw_hour_side );
		break;
	case 3:
		led( 9, segment <= ( _second_segment +
				NUM_SECOND_SEGMENTS * _subsecond_segment / NUM_SEGMENTS ) );
		led( 8, draw_tick );
		led( 7, draw_minute );
		led( 6, draw_minute || draw_minute_side );
		led( 5, draw_minute || draw_minute_side || draw_hour );
		for( int a = 0; a <= 4; a++ )
			led( a, draw_minute || draw_minute_side ||
					draw_hour || draw_hour_side );
		break;
	}
}


void AnalogueClockMode::draw_reset()
{
	int old_second_segment = _second_segment;

	// calculate hand locations
	_hour_segment = ( Time::get_hours() % 12 ) * 5 * NUM_SECOND_SEGMENTS +
		( 5 * NUM_SECOND_SEGMENTS * Time::get_minutes() / 60 );
	_minute_segment = Time::get_minutes() * NUM_SECOND_SEGMENTS +
		( NUM_SECOND_SEGMENTS * Time::get_seconds() / 60 );
	_second_segment = Time::get_seconds() * NUM_SECOND_SEGMENTS;

	// calculate the subsecond segment
	if( old_second_segment != _second_segment ) {
		_second_millis = millis();
		_subsecond_segment = 0;
	}
	else {
		unsigned long elapsed = ::millis() - _second_millis;
		_subsecond_segment = elapsed * NUM_SEGMENTS / 1000;
	}
}


void AnalogueClockMode::activate()
{
	_flavour = 0;
	_second_millis = ::millis();

	Text::reset();
}


void AnalogueClockMode::press()
{
	if( ++_flavour >= 4 )
		_flavour = 0;
}
