/*
 * analogue_clock_mode.h
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _ANALOGUE_CLOCK_MODE_H_
#define _ANALOGUE_CLOCK_MODE_H_


#include "mode.h"


class AnalogueClockMode : public Mode
{
public:

	/**
	 * Called once per segment to draw a column of LEDs.
	 *
	 * @param segment the segment number to draw
	 */
	void draw( int segment );

	/**
	 * Called at the start of a "frame", before any segments are drawn.
	 */
	void draw_reset();

	/**
	 * Called when the mode is becoming active.
	 */
	void activate();

	/**
	 * Called when the button has been pressed.
	 */
	void press();

private:

	/** the segment that the hour hand should be at */
	int _hour_segment;

	/** the segment that the minute hand should be at */
	int _minute_segment;

	/** the segment that the second hand should be at */
	int _second_segment;

	/** the segment that the subsecond hand should be at */
	int _subsecond_segment;

	/** the flavour */
	int _flavour;

	/** number of elapsed milliseconds this second */
	unsigned long _second_millis;
};


#endif //_ANALOGUE_CLOCK_MODE_H_
