/*
 * digital_clock_mode.cc
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "digital_clock_mode.h"
#include "common.h"
#include "text.h"
#include "text_renderer.h"
#include "time.h"
#include "Arduino.h"


void DigitalClockMode::draw_reset()
{
	switch( _flavour )
	{
	case 0: {
			PString str0( Text::_messages[ 0 ], MESSAGE_LEN * 2 );
			str0.format( "%d%c%02d %s", Time::get_hours() % 12,
					::millis() % 1000 < 500? ':' : ' ',
					Time::get_minutes(),
					Time::get_hours() >= 12? "pm" : "am" );
			Text::set_message_text( 0, str0 );
		}
		break;
	case 1: {
			PString str0( Text::_messages[ 0 ], MESSAGE_LEN * 2 );
			str0.format( "%d%s %s, %d", Time::get_day(),
					Time::get_day_suffix(),
					Time::get_month_name(), Time::get_year() );
			Text::set_message_text( 0, str0 );
		}
		break;
	case 2: {
			PString str0( Text::_messages[ 0 ], MESSAGE_LEN * 2 );
			PString str1( Text::_messages[ 2 ], MESSAGE_LEN * 2 );
			str0.format( "%d%c%02d %s", Time::get_hours() % 12,
					::millis() % 1000 < 500? ':' : ' ',
					Time::get_minutes(),
					Time::get_hours() >= 12? "pm" : "am" );
			str1.format( "%d.%02d.%d", Time::get_day(),
					Time::get_month(), Time::get_year() );
			Text::set_message_text( 0, str0 );
			Text::set_message_text( 2, str1 );
		}
		break;
	}
}


void DigitalClockMode::activate()
{
	_flavour = 0;

	reset_messages();
}


void DigitalClockMode::press()
{
	if( ++_flavour >= 3 )
		_flavour = 0;

	reset_messages();
}


void DigitalClockMode::reset_messages()
{
	Text::reset();

	switch( _flavour )
	{
	case 0:
		Text::set_up_message( 0, Text::MODE_TOP | Text::MODE_ALL );
		break;
	case 1:
		Text::set_up_message( 0, Text::MODE_HSCROLL | Text::MODE_TOP |
				Text::MODE_THREEQUARTERS );
		break;
	case 2:
		Text::set_up_message( 0, Text::MODE_TOP | Text::MODE_HALF,
				Text::SCALE_SMALL );
		Text::set_up_message( 2, Text::MODE_BOTTOM | Text::MODE_HALF,
				Text::SCALE_SMALL );
		break;
	}
}
